/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/logconn/def.hpp>
#include <ox/logconn/logconn.hpp>

#include <keel/media.hpp>

#include <nostalgia/modules/keelmodules.hpp>

#include "app.hpp"

static ox::Error run(int argc, const char **argv) noexcept {
#ifndef OX_BARE_METAL
	// GBA doesn't need the modules and calling this doubles the size of the
	// binary.
	nostalgia::registerKeelModules();
#endif
	if (argc < 2) {
		oxErr("Please provide path to project directory or OxFS file.\n");
		return OxError(1);
	}
	const auto path = argv[1];
	oxRequireM(fs, keel::loadRomFs(path));
	return run(std::move(fs));
}

#ifdef _WIN32
int WinMain() {
	auto const argc = __argc;
	auto const argv = const_cast<const char**>(__argv);
#else
int main(int argc, const char **argv) {
#endif
	OX_INIT_DEBUG_LOGGER(loggerConn, "Nostalgia Player")
	ox::Error err;
#ifdef __cpp_exceptions
	try {
		err = run(argc, argv);
	} catch (ox::Exception const&ex) {
		err = ex.toError();
	} catch (...) {
		err = OxError(1, "Non-Ox exception");
	}
#else
	err = run(argc, argv);
#endif
	oxAssert(err, "Something went wrong...");
	return static_cast<int>(err);
}
